import { shortcut, store } from '@core/core';
import { VtapeXtype, HtapeXtype, IconButtonXtype, TextValueComboXtype, ButtonXtype, TextEditorXtype, ButtonGroupXtype, VerticalXtype } from 'ui';
import { ListItem } from './list_item';
import { TableModelModel } from './table_list.model';
import { fineServletURL } from '@constants/env';

@shortcut()
@store(TableModelModel)
export class TableList extends BI.Widget {
    static xtype = 'dec.dcm.connection.plugin.redis.table_list'

    store: TableModelModel['store']
    model: TableModelModel['model']

    buttonGroup: any;
    connectionTextValue: any;
    searchText: any;

    props = {
        database: '',
    }

    watch = {
        connections: () => {
            this.connectionTextValue.populate(this.renderConnectionList());
            const name = this.model.connections.length > 0 ? this.model.connections[0] : '';
            this.connectionTextValue.setValue(name);
            this.store.setSelectedConnection(name);
        },
        'selectedConnection || search': () => {
            this.store.initTableList();
        },
        tables: (tables: string[]) => {
            this.buttonGroup.populate(this.renderTableList());
        },
    }

    render() {
        return {
            type: VtapeXtype,
            hgap: 10,
            bgap: 5,
            items: [{
                el: {
                    type: HtapeXtype,
                    items: [{
                        type: TextValueComboXtype,
                        ref: (_ref: any) => {
                            this.connectionTextValue = _ref;
                        },
                        items: [],
                        listeners: [{
                            eventName: BI.TextValueCombo.EVENT_CHANGE,
                            action: () => {
                                const name = this.connectionTextValue.getValue()[0];
                                this.store.setSelectedConnection(name);
                            },
                        }],
                    }, {
                        el: {
                            type: IconButtonXtype,
                            cls: 'redis-site-font',
                            title: BI.i18nText('Plugin-Redis_Data_Connection'),
                            handler: () => {
                                window.location.href = `${fineServletURL}#management/connection`;
                            },
                        },
                        width: 25,
                    }, {
                        el: {
                            type: IconButtonXtype,
                            cls: 'redis-refresh-font',
                            title: BI.i18nText('Plugin-Redis_Refresh'),
                            handler: () => {
                                this.store.initData();
                            },
                        },
                        width: 25,
                    }],
                },
                height: 25,
            }, {
                el: {
                    type: HtapeXtype,
                    items: [{
                        type: TextEditorXtype,
                        height: 24,
                        watermark: BI.i18nText('Plugin-Redis_Keys_Pattern'),
                        ref: (_ref: any) => {
                            this.searchText = _ref;
                        },
                    }, {
                        el: {
                            type: ButtonXtype,
                            minWidth: 50,
                            text: BI.i18nText('Plugin-Redis_Keys_Pattern_Search'),
                            handler: () => {
                                this.store.setSearch(this.searchText.getValue());
                            },
                        },
                        width: 50,
                    }],
                },
                height: 25,
            }, {
                type: VtapeXtype,
                items: [{
                    type: ButtonGroupXtype,
                    chooseType: BI.Selection.None,
                    layouts: [{
                        type: VerticalXtype,
                    }],
                    items: [],
                    ref: (_ref: any) => {
                        this.buttonGroup = _ref;
                    },
                }],
            }],
        };
    }

    private renderConnectionList() {
        return this.model.connections.map(item => {
            return {
                text: item,
                value: item,
            };
        });
    }

    private renderTableList() {
        return this.model.tables.map(item => {
            return {
                type: ListItem.xtype,
                text: item,
                value: item,
            };
        });
    }

    mounted() {
        this.store.initData();
    }

    public getSelectedDatabase() {
        return this.model.selectedConnection;
    }
}
